.SECONDEXPANSION:

ifndef ALL_DEPS
ALL_DEPS=$(shell python3 build/dependencies.py dependencies)
endif

GIT_PATH=git@bsgitlab.bs.informatik.uni-siegen.de:scoutcomp
GIT_POSTFIX=.git

BUILD_DIR=$(CURDIR)/build
TARGET_DIR=$(BUILD_DIR)/classes
DOC_DIR=$(BUILD_DIR)/doc
SOURCE_DIR=src

DEP_DIRS=$(addprefix ../, $(ALL_DEPS))

DEPS_PRE_BUILD=$(addprefix pre., $(ALL_DEPS))
DEPS_POST_BUILD=$(addprefix post., $(ALL_DEPS))
DEPS_CLEAN=$(addprefix clean., $(ALL_DEPS))
DEPS_PULL=$(addprefix pull., $(ALL_DEPS))

BUILD_TARGET=$(if $(findstring -j,$(MAKEFLAGS)), build_parallel, build_sequential)
build: $$(BUILD_TARGET)

build_sequential: $(DEP_DIRS) pre_build javac post_build

build_parallel:
	@$(MAKE) --no-print-directory $(DEP_DIRS) ALL_DEPS="$(ALL_DEPS)"
	@$(MAKE) --no-print-directory pre_build ALL_DEPS="$(ALL_DEPS)"
	@$(MAKE) --no-print-directory javac ALL_DEPS="$(ALL_DEPS)"
	@$(MAKE) --no-print-directory post_build ALL_DEPS="$(ALL_DEPS)"

javac: OUTDATED_FILES=$(shell python3 $(BUILD_DIR)/dependencies.py outdated)
javac: $(TARGET_DIR)
	@echo building $(words $(OUTDATED_FILES)) outdated file\(s\)...
	@if [ ! -z "$(OUTDATED_FILES)" ]; then\
		javac -cp $(TARGET_DIR) -sourcepath $(SOURCE_DIR) -d $(TARGET_DIR) $(OUTDATED_FILES);\
	fi

doc: SOURCE_FILES=$(shell python3 $(BUILD_DIR)/dependencies.py all)
doc:
	javadoc -cp $(TARGET_DIR) -sourcepath $(SOURCE_DIR) -d $(DOC_DIR) $(SOURCE_FILES)

jar: build
ifneq ($(and $(MAIN_CLASS),$(JAR_NAME)),)
	cd build/classes && jar -cvfe ../$(JAR_NAME) $(MAIN_CLASS) .
else
	$(error please sepcify MAIN_CLASS and JAR_NAME variables to create a jar file)
endif

pull: $(DEPS_PULL)
	@echo >/dev/null

$(TARGET_DIR)/%.class: $(SOURCE_DIR)/%.java
	$(eval OUTDATED_FILES+=$<)

../%:
ifeq ($(CLONE),true)
	git clone $(GIT_PATH)/$*$(GIT_POSTFIX) $@
else
	$(warning directory "$@" not found.)
	$(error please manually clone the corresponding repository or rerun "make" with "CLONE=true")
endif

$(TARGET_DIR):
	mkdir $@

pull.%:
	git -C ../$* pull

pre.%:
	@$(MAKE) --no-print-directory -C ../$* pre_build ALL_DEPS= BUILD_DIR=$(BUILD_DIR)

post.%:
	@$(MAKE) --no-print-directory -C ../$* post_build ALL_DEPS= BUILD_DIR=$(BUILD_DIR)

clean.%:
	@$(MAKE) --no-print-directory -C ../$* clean ALL_DEPS= BUILD_DIR=$(BUILD_DIR)

pre_build:: $(DEPS_PRE_BUILD)
	@echo >/dev/null

post_build:: $(DEPS_POST_BUILD)
	@echo >/dev/null

clean:: $(DEPS_CLEAN)
	@echo "cleaning $(shell pwd)"
	rm -rf $(TARGET_DIR) $(DOC_DIR) build/*.jar
