(defvar cola-mode-hook nil)

(defvar cola-mode-map
  (let ((map (make-keymap)))
    (define-key map "\C-j" 'newline-and-indent)
    map)
  "Keymap for Cola major mode")

;;;###autoload
(add-to-list 'auto-mode-alist '("\\.cola\\'" . cola-mode))

(defconst cola-font-lock-keywords-1
  (list
   '("\\<\\(unit\\|import\\)\\>" . font-lock-keyword-face)
   '("\\<\\(class\\|fun\\|static\\|typedef\\|va[lr]\\)\\>" . font-lock-keyword-face)
   '("\\<\\(if\\|else\\|switch\\|case\\|return\\|while\\|for\\|try\\|catch\\|new\\)\\>" . font-lock-keyword-face)
   '("\\<\\(private\\|extern\\|optional\\)\\>" . font-lock-keyword-face)
   '("\\('\\w*'\\)" . font-lock-variable-name-face))
  "Minimal highlighting expressions for Cola mode")

(defconst cola-font-lock-keywords-2
  (append cola-font-lock-keywords-1
		  (list
		   '("\\<\\(Any\\|bool\\|int\\|int8\\|int16\\|int32\\|int64\\|byte\\|String\\|double\\|float\\)\\>" . font-lock-type-face)
		   '("\\<\\(true\\|false\\)\\>" . font-lock-constant-face)
           ))
  "Additional Keywords/Sections to highlight in Cola mode")

(defvar cola-font-lock-keywords cola-font-lock-keywords-2
  "Default highlighting expressions for Cola mode")

(defvar cola-mode-syntax-table
  (let ((st (make-syntax-table)))
    (modify-syntax-entry ?_ "w" st)
    (modify-syntax-entry ?/ ". 124b" st)
    (modify-syntax-entry ?* ". 23" st)
    (modify-syntax-entry ?\n "> b" st)
    st)
  "Syntax table for cola-mode")

(setq comment-start "//")
(setq comment-end "")

(defun cola-indent-line ()
  "Indent current line as COLA code"
  (interactive)
  (beginning-of-line)
  (if (bobp)
      (indent-line-to 0)
    (let ((not-indented t) cur-indent)
      (if (looking-at "^[ \t]*}[ \t]*")
          (progn
            (save-excursion
              (forward-line -1)
              (setq cur-indent (- (current-indentation) default-tab-width)))
            (if (< cur-indent 0)
                (setq cur-indent 0)))
        (save-excursion 
          (while not-indented
            (forward-line -1)
            (if (or (looking-at "^[ \t]*}[ \t]*$") (looking-at ".*;[ \t]*$")) ; Check for rule 3
                (progn
                  (setq cur-indent (current-indentation))
                  (setq not-indented nil))
              (if (looking-at "^.*{[ \t]*$")
                  (progn
                    (setq cur-indent (+ (current-indentation) default-tab-width))
                    (setq not-indented nil))
                (if (bobp) ; Check for rule 5
                    (setq not-indented nil)))))))
      (if cur-indent
          (indent-line-to cur-indent)
        (indent-line-to 0)))))

(defun cola-mode ()
  "Major mode for editing Component Language files"
  (interactive)
  (kill-all-local-variables)
  (use-local-map cola-mode-map)
  (set-syntax-table cola-mode-syntax-table)
  (set (make-local-variable 'indent-line-function) 'cola-indent-line)  
  (set (make-local-variable 'font-lock-defaults) '(cola-font-lock-keywords))
  (setq major-mode 'cola-mode)
  (setq mode-name "COLA")
  (setq comment-use-syntax 't)
  (run-hooks 'cola-mode-hook))

(provide 'cola-mode)
