/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast;

import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * This class represents an implementation (aka class).
 */
public class ClassNode extends Node {
    private final String name;
    private final List<MethodNode> methods;
    private final List<AttributeNode> attributes;
    private final List<String> superClassNames;

    /**
     * Constructs a new implementation.
     *
     * @param name the implementations name
     * @param methods list of methods that belong to this implementation
     * @param attributes list of attributes that belong to this implementation
     * @param superClassNames names of extended or implemented classes
     */
    public ClassNode(String name,
                     List<MethodNode> methods,
                     List<AttributeNode> attributes,
                     List<String> superClassNames) {
        this.name = name;
        this.methods = methods;
        this.attributes = attributes;
        this.superClassNames = superClassNames;
    }

    public String getName() {
        return name;
    }

    public boolean hasSuperclass() {
        return (superClassNames != null) && (superClassNames.size() > 0);
    }

    public List<String> getSuperclasses() {
        return superClassNames;
    }

    public List<AttributeNode> getAttributeNodes() {
        return attributes;
    }

    public List<MethodNode> getMethods() {
        return methods;
    }

    public <T> void accept(VoidVisitor<T> visitor, T arg) {
        visitor.visit(this, arg);
    }

    public <T,R> R accept(GenericVisitor<T,R> visitor, T arg) {
        return visitor.visit(this, arg);
    }
}
