/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast;

import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

import java.util.List;
import java.util.Set;

/**
 * This class represents a component.
 */
public class ComponentNode extends Node {
    public enum UnitType {
        Component,
        Module
    }
    private final List<ClassNode> classes;
    private final List<TypedefNode> typedefs;
    private final Set<String> dependencies;
    private final String name;
    private final UnitType unitType;

    /**
     * Constructs a new component.
     *
     * @param classes a list of implementations
     * @param typedefs a list of typedefs
     */
    public ComponentNode(String name,
                         List<ClassNode> classes,
                         List<TypedefNode> typedefs,
                         Set<String> dependencies,
                         UnitType unitType) {
        this.name = name;
        this.classes = classes;
        this.typedefs = typedefs;
        this.dependencies = dependencies;
        this.unitType = unitType;
    }

    public String getComponentClassName() {
        if (!isComponent())
            return "";
        return name.substring(name.lastIndexOf('.')+1);
    }

    public boolean isComponent() {
        return unitType == UnitType.Component;
    }

    public String getName() {
        return name;
    }

    public List<ClassNode> getClasses() {
        return classes;
    }

    public List<TypedefNode> getTypedefs() {
        return typedefs;
    }

    public Set<String> getDependencies() {
        return dependencies;
    }

    public <T> void accept(VoidVisitor<T> visitor, T arg) {
        visitor.visit(this, arg);
    }

    public <T,R> R accept(GenericVisitor<T,R> visitor, T arg) {
        return visitor.visit(this, arg);
    }
}
