/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.      If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast;

import cola.ast.type.AbstractType;
import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

/**
 * This class represents a typed identifier, i.e pair of name and
 * type.
 */
public class IdentifierWithType extends Node {
    private final String name;
    private final AbstractType type;
    private final boolean isSafe;

    /**
     * Constructs a new typed identifier.
     *
     * @param name the identifiers name
     * @param type the identifiers type
     */
    public IdentifierWithType(String name, AbstractType type, boolean isSafe) {
        this.name = name;
        this.type = type;
        this.isSafe = isSafe;
    }

    public String getName() {
        return name;
    }

    public AbstractType getType() {
        return type;
    }

    public boolean isSafe() {
        return isSafe;
    }

    public <T> void accept(VoidVisitor<T> visitor, T arg) {
        visitor.visit(this, arg);
    }

    public <T,R> R accept(GenericVisitor<T,R> visitor, T arg) {
        return visitor.visit(this, arg);
    }
}
