/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.exp;

import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

public class FieldAccessExp extends Exp {
    private final Exp target;
    private final IdExp field;
    
    public FieldAccessExp(Exp target, IdExp field) {
        this.target = target;
        this.field = field;
    }

    public Exp getTarget() {
        return target;
    }

    public IdExp getField() {
        return field;
    }
    
    public <A> void accept(VoidVisitor<A> visitor, A arg) {
        visitor.visit(this, arg);
    }

    public <A,R> R accept(GenericVisitor<A,R> visitor, A arg) {
        return visitor.visit(this, arg);
    }
}
