/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.exp;

import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

public class LiteralExp extends Exp {
    private final String svalue;
    private final Integer ivalue;
    private final Float fvalue;
    private final Double dvalue;
    private final Boolean bvalue;
    private final boolean isChar;

    public LiteralExp(String s, boolean isChar) {
        this(null, null, null, null, s, isChar);
    }

    public LiteralExp(Float f) {
        this(null, f, null, null, null, false);
    }

    public LiteralExp(Double d) {
        this(null, null, d, null, null, false);
    }

    public LiteralExp(Integer i) {
        this(i, null, null, null, null, false);
    }

    public LiteralExp(Boolean b) {
        this(null, null, null, b, null, false);
    }

    public LiteralExp() {
        this(null, null, null, null, null, false);
    }

    private LiteralExp(Integer i, Float f, Double d, Boolean b, String s, boolean isChar) {
        this.ivalue = i;
        this.fvalue = f;
        this.dvalue = d;
        this.bvalue = b;
        this.svalue = s;
        this.isChar = isChar;
    }

    public Integer getIntegerValue() {
        return ivalue;
    }

    public Float getFloatValue() {
        return fvalue;
    }

    public Double getDoubleValue() {
        return dvalue;
    }

    public Boolean getBooleanValue() {
        return bvalue;
    }

    public String getStringValue() {
        return svalue;
    }

    public boolean isChar() {
        return (svalue != null) && isChar;
    }

    public boolean isInteger() {
        return ivalue != null;
    }

    public boolean isFloat() {
        return fvalue != null;
    }

    public boolean isDouble() {
        return dvalue != null;
    }

    public boolean isString() {
        return (svalue != null) && !isChar;
    }

    public boolean isBool() {
        return bvalue != null;
    }

    public boolean isNull() {
        return (ivalue == null)
            && (fvalue == null)
            && (dvalue == null)
            && (bvalue == null)
            && (svalue == null);
    }

    public <A> void accept(VoidVisitor<A> visitor, A arg) {
        visitor.visit(this, arg);
    }

    public <A,R> R accept(GenericVisitor<A,R> visitor, A arg) {
        return visitor.visit(this, arg);
    }
}
