/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.stm;

import cola.ast.exp.Exp;
import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Represents a switch statement.
 */
public class SwitchStm extends Stm {
    private final Exp switchExp;
    private final Map<Exp, Stm> cases;
    private final Stm defaultCase;
    
    public SwitchStm(Exp switchExp, Stm defaultCase) {
        this.switchExp = switchExp;
        cases = new LinkedHashMap<>();
        this.defaultCase = defaultCase;
    }

    public Exp getSwitchExp() {
        return switchExp;
    }

    public Map<Exp, Stm> getCases() {
        return cases;
    }

    public Stm getDefaultCase() {
        return defaultCase;
    }
    
    public <A> void accept(VoidVisitor<A> visitor, A arg) {
        visitor.visit(this, arg);
    }

    public <A,R> R accept(GenericVisitor<A,R> visitor, A arg) {
        return visitor.visit(this, arg);
    }
}
