/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.stm;

import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

/**
 * This class represents a try-catch clause
 */
public class TryCatchStm extends Stm {
    private final Stm tryStm;
    private final Stm catchStm;

    public TryCatchStm(Stm tryStm, Stm catchStm) {
        this.tryStm = tryStm;
        this.catchStm = catchStm;
    }

    public Stm getTryStm() {
        return tryStm;
    }

    public Stm getCatchStm() {
        return catchStm;
    }

    public <A> void accept(VoidVisitor<A> visitor, A arg) {
        visitor.visit(this, arg);
    }

    public <A,R> R accept(GenericVisitor<A,R> visitor, A arg) {
        return visitor.visit(this, arg);
    }
    
}
