/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.stm;

import cola.ast.exp.Exp;
import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

/**
 * Represents a loop: while e do s;
 */
public class WhileStm extends Stm {
    private final Exp condition;
    private final Stm body;
    
    public WhileStm(Exp condition, Stm body) {
        this.condition = condition;
        this.body = body;
    }

    public Exp getCondition() {
        return condition;
    }

    public Stm getBody() {
        return body;
    }
    
    public <A> void accept(VoidVisitor<A> visitor, A arg) {
        visitor.visit(this, arg);
    }

    public <A,R> R accept(GenericVisitor<A,R> visitor, A arg) {
        return visitor.visit(this, arg);
    }
}
