/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.type;

/**
 * Base class for types.
 */
public abstract class AbstractType extends cola.ast.Node {
    protected int assertions = 0;

    public String assertionsToString() {
        String res = ":";
        if (isLocal()) res += "l";
        if (isFinal()) res += "f";
        if (isIntimate()) res += "I";
        if (isTransient()) res += "T";
        return res.equals(":") ? "" : res;
    }
    
    public int getAssertions() {
        return assertions;
    }

    public void setAssertions(int assertions) {
        this.assertions = assertions;
    }

    public boolean isLocal() {
        return (assertions & 1) != 0;
    }

    public boolean isIntimate() {
        return (assertions & 2) != 0;
    }

    public boolean isFinal() {
        return (assertions & 4) != 0;
    }

    public boolean isTransient() {
        return (assertions & 8) != 0;
    }

    public abstract String getPrettyName();

    public abstract String getInternalName();

    public abstract AbstractType asLocal();
}
