/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.type;

import cola.ast.visitor.GenericVisitor;
import cola.ast.visitor.VoidVisitor;

/**
 * This class represents a named type.
 */
public class TypeIdentifier extends AbstractType {
    private final String name;

    /**
     * Constructs a new type with given name.
     *
     * @param name the name
     */
    public TypeIdentifier(String name) {
        this.name = name;
    }

    public <T> void accept(VoidVisitor<T> visitor, T arg) {
        visitor.visit(this, arg);
    }

    public <T,R> R accept(GenericVisitor<T,R> visitor, T arg) {
        return visitor.visit(this, arg);
    }

    public String getName() {
        return name;
    }

    public String getPrettyName() {
        return name;
    }

    public String getInternalName() {
        return name + assertionsToString();
    }

    @Override
    public boolean equals(Object other) {
        if (other instanceof TypeIdentifier) {
            TypeIdentifier t = (TypeIdentifier) other;
            return name.equals(t.name)
                && (assertions == t.assertions);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return name.hashCode() * 91 + assertions;
    }

    public TypeIdentifier asLocal() {
        if (!isLocal()) {
            TypeIdentifier local = new TypeIdentifier(name);
            local.setAssertions(assertions+1);
            return local;
        }
        return this;
    }
}
