/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.visitor;

import cola.ast.*;
import cola.ast.exp.*;
import cola.ast.stm.*;
import cola.ast.type.*;

public abstract class GenericVisitor<T,R> {
    public abstract R visit(AttributeNode node, T arg);
    public abstract R visit(ComponentNode node, T arg);
    public abstract R visit(ClassNode node, T arg);
    public abstract R visit(MethodNode node, T arg);
    public abstract R visit(TypedefNode node, T arg);
    public abstract R visit(IdentifierWithType node, T arg);

    public abstract R visit(ArrayType node, T arg);
    public abstract R visit(FunctionalType node, T arg);
    public abstract R visit(TypeIdentifier node, T arg);
    public abstract R visit(SimpleType node, T arg);
    public abstract R visit(StructuralType node, T arg);

    public abstract R visit(BlockStm node, T arg);
    public abstract R visit(ExpStm node, T arg);
    public abstract R visit(ReturnStm node, T arg);
    public abstract R visit(AssignStm node, T arg);
    public abstract R visit(VarStm node, T arg);
    public abstract R visit(IfStm node, T arg);
    public abstract R visit(WhileStm node, T arg);
    public abstract R visit(ForStm node, T arg);
    public abstract R visit(BreakStm node, T arg);
    public abstract R visit(ContinueStm node, T arg);
    public abstract R visit(InvokeStm node, T arg);
    public abstract R visit(AsmStm ndoe, T arg);
    public abstract R visit(TryCatchStm node, T arg);
    public abstract R visit(SwitchStm node, T arg);
    
    public abstract R visit(IdExp node, T arg);
    public abstract R visit(LiteralExp node, T arg);
    public abstract R visit(UnaryExp node, T arg);
    public abstract R visit(EnclosedExp node, T arg);
    public abstract R visit(BinaryExp node, T arg);
    public abstract R visit(ConditionalExp node, T arg);
    public abstract R visit(CallExp node, T arg);
    public abstract R visit(ThisExp node, T arg);
    public abstract R visit(NewExp node, T arg);
    public abstract R visit(NewArrayExp node, T arg);
    public abstract R visit(FieldAccessExp node, T arg);
    public abstract R visit(TypeVerdictExp node, T arg);
    public abstract R visit(IncreaseOrDecreaseExp node, T arg);
    public abstract R visit(AnonFunctionExp node, T arg);
}
