/*
 * COSMA -- A Virtual Machine for the Secure Cooperation of Untrusted Components.
 * Copyright (C) 2019  Roland Wismüller and Damian Ludwig, University of Siegen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package cola.ast.visitor;

import cola.ast.*;
import cola.ast.exp.*;
import cola.ast.stm.*;
import cola.ast.type.*;

public abstract class VoidVisitor<T> {
    public abstract void visit(AttributeNode node, T arg);
    public abstract void visit(ComponentNode node, T arg);
    public abstract void visit(ClassNode node, T arg);
    public abstract void visit(MethodNode node, T arg);
    public abstract void visit(TypedefNode node, T arg);
    public abstract void visit(IdentifierWithType node, T arg);

    public abstract void visit(ArrayType node, T arg);
    public abstract void visit(FunctionalType node, T arg);
    public abstract void visit(TypeIdentifier node, T arg);
    public abstract void visit(SimpleType node, T arg);
    public abstract void visit(StructuralType node, T arg);

    public abstract void visit(BlockStm node, T arg);
    public abstract void visit(ExpStm node, T arg);
    public abstract void visit(ReturnStm node, T arg);
    public abstract void visit(AssignStm node, T arg);
    public abstract void visit(VarStm node, T arg);
    public abstract void visit(IfStm node, T arg);
    public abstract void visit(WhileStm node, T arg);
    public abstract void visit(ForStm node, T arg);
    public abstract void visit(BreakStm node, T arg);
    public abstract void visit(ContinueStm node, T arg);
    public abstract void visit(InvokeStm node, T arg);
    public abstract void visit(AsmStm node, T arg);
    public abstract void visit(TryCatchStm node, T arg);
    public abstract void visit(SwitchStm node, T arg);
        
    public abstract void visit(IdExp node, T arg);
    public abstract void visit(LiteralExp node, T arg);
    public abstract void visit(UnaryExp node, T arg);
    public abstract void visit(EnclosedExp node, T arg);
    public abstract void visit(BinaryExp node, T arg);
    public abstract void visit(ConditionalExp node, T arg);
    public abstract void visit(CallExp node, T arg);
    public abstract void visit(ThisExp node, T arg);
    public abstract void visit(NewExp node, T arg);
    public abstract void visit(NewArrayExp node, T arg);
    public abstract void visit(FieldAccessExp node, T arg);
    public abstract void visit(TypeVerdictExp node, T arg);
    public abstract void visit(IncreaseOrDecreaseExp node, T arg);
    public abstract void visit(AnonFunctionExp node, T arg);
}
