package cola.compiler;

import cosma.bytecode.builder.ClassBuilder;
import cosma.tools.logger.LogManager;
import cosma.tools.logger.Logger;

import java.util.HashMap;
import java.util.Map;

public abstract class ClassCache {
    private static final Logger logger = LogManager.getCurrentClassLogger();
    private static final Map<String, ClassBuilder> classes = new HashMap<>();
    private static boolean __committed__ = false;
    
    public static void addClass(String className, ClassBuilder classBuilder) {
        ////
        assert !__committed__ : "ClassCache.addClass(): Add after commit";
        ////
        classes.put(className, classBuilder);
        logger.debug("Class added: {class}", className);
    }

    public static ClassBuilder getClassForName(String name) {
        ClassBuilder classBuilder = classes.get(name);
        /////
        if (__committed__ && (classBuilder == null)) {
            logger.error("Class not found: {name}", name);
            System.exit(1);
        }
        /////
        return classes.get(name);
    }

    public static void commit() {
        __committed__ = true;
    }

    public static void printStats() {
        logger.debug("ClassCache: {numClasses} classes registered", classes.size());
    }
}
