package cola.compiler;

import cola.ast.ClassNode;
import cola.ast.visitor.VoidVisitorAdapter;
import cosma.bytecode.ClassModifier;
import cosma.bytecode.builder.Builder;
import cosma.bytecode.builder.ClassBuilder;
import cosma.bytecode.builder.IntoType;

import java.util.EnumSet;

class ClassGenerator extends VoidVisitorAdapter<ClassBuilder> {
    private final Builder componentBuilder;

    ClassGenerator(Builder componentBuilder) {
        this.componentBuilder = componentBuilder;
    }
    
    public void visit(ClassNode node, ClassBuilder cb) {
        String className = node.getName();
        ClassBuilder classBuilder = getClassForName(className);

        if (node.hasSuperclass()) {
            for (String superClassName : node.getSuperclasses()) {
                ClassBuilder superClassBuilder = getClassForName(superClassName);
                classBuilder.addSuperClass(superClassBuilder);
            }
        }

        IntoType classType = classBuilder.getClassType();
        TypeCache.addType(className, classType);
    }

    private ClassBuilder getClassForName(String className) {
        ClassBuilder cb = ClassCache.getClassForName(className);
        if (cb == null) {
            cb = componentBuilder.createClass(className, EnumSet.allOf(ClassModifier.class));
            ClassCache.addClass(className, cb);
        }
        return cb;
    }
}
