package cola.compiler;

import cola.ast.MethodNode;
import cola.ast.ModifierNode;
import cola.ast.visitor.VoidVisitorAdapter;
import cosma.bytecode.builder.Builder;
import cosma.bytecode.builder.ClassBuilder;
import cosma.bytecode.builder.IntoType;
import cosma.bytecode.builder.MethodBuilder;
import cosma.bytecode.methodinfo.MethodAccessFlags;

import java.util.EnumSet;

public class MethodGenerator extends VoidVisitorAdapter<ClassBuilder> {
    private final TypeGenerator typeGenerator;

    MethodGenerator(Builder componentBuilder) {
        this.typeGenerator = new TypeGenerator(componentBuilder);
    }
    
    public void visit(MethodNode node, ClassBuilder classBuilder) {
        String methodName = node.getName();
        EnumSet<MethodAccessFlags> accessFlags = EnumSet.noneOf(MethodAccessFlags.class);
        if (methodName.equals("$init")) {
            accessFlags.add(MethodAccessFlags.Constructor);
        } else if (methodName.equals("$delete")) {
            accessFlags.add(MethodAccessFlags.Destructor);
        } else if (node.isStatic()) {
            accessFlags.add(MethodAccessFlags.Static);
        }
        if (node.getModifier() == ModifierNode.PROTECTED)
            accessFlags.add(MethodAccessFlags.Protected);
        if (node.getModifier() == ModifierNode.PRIVATE)
            accessFlags.add(MethodAccessFlags.Private);
        
        MethodBuilder methodBuilder = classBuilder.createMethod(methodName, accessFlags);
        if (node.hasParameters()) {
            node.getParameters().forEach(v -> {
                    IntoType type = v.getType().accept(typeGenerator, null);
                    /////
                    assert type != null : "Type is null";
                    /////
                    String name = v.getName();
                    methodBuilder.createArgument(type, name);
                });
        }
        if (node.hasResults()) {
            node.getResultTypes().forEach(v -> {
                    IntoType type = v.accept(typeGenerator, null);
                    /////
                    assert type != null : "Type is null";
                    /////
                    methodBuilder.addReturnType(type);
                });
        }
    }
}
