package cola.compiler;

import cosma.bytecode.builder.IntoType;
import cosma.tools.logger.LogManager;
import cosma.tools.logger.Logger;

import java.util.HashMap;
import java.util.Map;

public abstract class TypeCache {
    private static final Logger logger = LogManager.getCurrentClassLogger();
    private static final Map<String, IntoType> types = new HashMap<>();
    private static boolean __committed__ = false;

    public static void addType(String typeName, IntoType type) {
        /////
        assert !__committed__ : "TypeInfo.addTypeBuilder(): Add after commit";
        /////
        types.put(typeName, type);
        logger.debug("Type added: {name}", typeName);
    }

    public static IntoType getTypeForName(String name) {
        IntoType res = types.get(name);
        /////
        if (__committed__ && (res == null)) {
            logger.error("Type not found: {name}", name);
            System.exit(1);
        }
        /////
        return types.get(name);
    }

    public static void commit() {
        __committed__ = true;
    }

    public static void printStats() {
        logger.debug("TypeCache: {numTypes} types registered", types.size());
    }
}
